/*******************************************************************************
 * Copyright (c) 2008 Marian Schedenig
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marian Schedenig - initial API and implementation
 *******************************************************************************/ 

package com.musgit.eclipse.grepconsole.adapters;

import java.util.List;
import java.util.regex.Pattern;

import org.eclipse.swt.graphics.Color;

import com.musgit.eclipse.grepconsole.Activator;
import com.musgit.eclipse.grepconsole.model.GrepExpression;
import com.musgit.eclipse.grepconsole.model.GrepStyle;

/**
 * An adapter class for grep expressions. Unlike the pure model GrepExpression
 * class, adapter instances hold handles to system resources like colours.
 *  
 * @author msched
 */
public class GrepExpressionAdapter
{
	/** Backlink to the plugin instance which created this adapter. */
	private Activator plugin;
	
	/** Backlink to the original grep expression instance. */
	private GrepExpression expression;
	
	/** Grep pattern created from the epxression. */
	private Pattern pattern;
	
	/** Array of foreground colours, taken from the expression's styles. */
	private Color[] foregrounds;
	
	/** Array of background colours, taken from the expression's styles. */
	private Color[] backgrounds;
	
	/** Array of font styles, taken from the expression's styles. */
	private int[] fontStyles;

	/** Array of underline flags, taken from the expression's styles. */
	private boolean[] underlines;

	/** Array of underline colours, taken from the expression's styles. */
	private Color[] underlineColors;

	/** Array of strikeout flags, taken from the expression's styles. */
	private boolean[] strikeouts;

	/** Array of strikeout colours, taken from the expression's styles. */
	private Color[] strikeoutColors;

	/** Array of border flags, taken from the expression's styles. */
	private boolean[] borders;

	/** Array of border colours, taken from the expression's styles. */
	private Color[] borderColors;

	/** Array of rise values, taken from the expression's styles. */
	private int[] rises;
	
	/**
	 * Creates a new adapter.
	 * 
	 * @param plugin Plugin creating the adapter.
	 * @param expression Expression to be represented by the adapter.
	 */
	public GrepExpressionAdapter(Activator plugin, GrepExpression expression)
	{
		this.plugin = plugin;
		this.expression = expression;
		
		refresh();
	}

	/**
	 * Refreshes the adapter's values from the grep expression.
	 */
	private void refresh()
	{
		pattern = Pattern.compile(expression.getGrepExpression());
		
		List<GrepStyle> styles = expression.getStyles();
		int count = styles.size();
		
		foregrounds = new Color[count];
		backgrounds = new Color[count];
		fontStyles = new int[count];
		underlines = new boolean[count];
		underlineColors = new Color[count];
		strikeouts = new boolean[count];
		strikeoutColors = new Color[count];
		borders = new boolean[count];
		borderColors = new Color[count];
		rises = new int[count];
		
		int i = 0;
		
		for(GrepStyle style: styles)
		{
			foregrounds[i] = plugin.getColor(style.getForeground());
			backgrounds[i] = plugin.getColor(style.getBackground());
			fontStyles[i] = style.getFontStyle();
			underlines[i] = style.isUnderline();
			underlineColors[i] = plugin.getColor(style.getUnderlineColor());
			strikeouts[i] = style.isStrikeout();
			strikeoutColors[i] = plugin.getColor(style.getStrikeoutColor());
			borders[i] = style.isBorder();
			borderColors[i] = plugin.getColor(style.getBorderColor());
			rises[i] = style.getRise();
			
			i++;
		}
	}
	
	public Pattern getPattern()
	{
		return pattern;
	}
	
	public int getGroupCount()
	{
		return foregrounds.length;
	}
	
	public Color getForeground(int i)
	{
		return foregrounds[i];
	}
	
	public Color getBackground(int i)
	{
		return backgrounds[i];
	}
	
	public int getFontStyle(int i)
	{
		return fontStyles[i];
	}
	
	public boolean isEnabled()
	{
		return expression.isEnabled();
	}

	public boolean getUnderline(int i)
	{
		return underlines[i];
	}

	public Color getUnderlineColor(int i)
	{
		return underlineColors[i];
	}

	public boolean getStrikeout(int i)
	{
		return strikeouts[i];
	}

	public Color getStrikeoutColor(int i)
	{
		return strikeoutColors[i];
	}

	public boolean getBorder(int i)
	{
		return borders[i];
	}

	public Color getBorderColor(int i)
	{
		return borderColors[i];
	}

	public int getRise(int i)
	{
		return rises[i];
	}
}
