/*******************************************************************************
 * Copyright (c) 2008 Marian Schedenig
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marian Schedenig - initial API and implementation
 *******************************************************************************/ 

package com.musgit.eclipse.grepconsole.manageexpressions;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;

import com.musgit.eclipse.grepconsole.Activator;
import com.musgit.eclipse.grepconsole.editexpression.EditExpressionDialog;
import com.musgit.eclipse.grepconsole.model.GrepConsoleSettings;
import com.musgit.eclipse.grepconsole.model.GrepExpression;
import com.musgit.eclipse.grepconsole.model.GrepStyle;
import com.musgit.eclipse.grepconsole.util.RGBUtil;

/**
 * A dialog for managing grep expressions.
 * 
 * @author msched
 */
public class ManageExpressionsDialog extends Dialog
{
	/** ID of the Apply button. */
	private static final int APPLY_ID = -1;

	/** Settings managed by this dialog. */
	private GrepConsoleSettings settings;
	
	/** GUI elements. */
	private Table table;
	private Button btnAdd;
	private Button btnEdit;
	private Button btnRemove;
	private Button btnUp;
	private Button btnDown;
	private CheckboxTableViewer viewer;
	private Button btnCopy;

	/**
	 * Creates a new dialog.
	 * 
	 * @param parent Parent shell.
	 */
	public ManageExpressionsDialog(Shell parent)
	{
		super(parent);

		Activator plugin = Activator.getDefault();
		this.settings = plugin.getSettings().clone();
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.window.Window#getShellStyle()
	 */
	@Override
	protected int getShellStyle()
	{
		return super.getShellStyle() | SWT.RESIZE;
		
		/*
		 * The following would create a non-modal dialog:
		 * return (super.getShellStyle() | SWT.RESIZE) & ~SWT.APPLICATION_MODAL;
		 */
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
	 */
	@Override
	protected void configureShell(Shell newShell)
	{
		super.configureShell(newShell);
		
		newShell.setText("Manage Grep Expressions");
	}

	/**
	 * Returns the settings instance being edited.
	 * 
	 * @return Settings instance.
	 */
	public GrepConsoleSettings getSettings()
	{
		return settings;
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	protected void createButtonsForButtonBar(Composite parent)
	{
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
		createButton(parent, APPLY_ID, "Apply", false);
		createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected Control createDialogArea(Composite parent)
	{
		Composite composite = new Composite(parent, SWT.NONE);
		composite.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		GridLayout layout = new GridLayout(2, false);
		composite.setLayout(layout);

		table = new Table(composite, SWT.BORDER | SWT.MULTI | SWT.H_SCROLL |
				SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.CHECK);
		table.setLinesVisible(true);
		table.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 6));
		table.addKeyListener(new KeyAdapter()
		{
			@Override
			public void keyPressed(KeyEvent e)
			{
				if(e.character == SWT.DEL || e.character == SWT.BS)
				{
					removeExpressions();
				}
				else if(e.keyCode == SWT.ARROW_UP && (e.stateMask & SWT.CTRL) != 0)
				{
					moveExpressionsUp();
				}
				else if(e.keyCode == SWT.ARROW_DOWN && (e.stateMask & SWT.CTRL) != 0)
				{
					moveExpressionsDown();
				}
			}
		});
		
		viewer = new CheckboxTableViewer(table);
		viewer.setContentProvider(new ExpressionsContentProvider());
		viewer.setLabelProvider(new ExpressionsLabelProvider());
		viewer.setInput(settings);
		
		viewer.addSelectionChangedListener(new ISelectionChangedListener()
		{
//			@Override
			public void selectionChanged(SelectionChangedEvent event)
			{
				refreshButtons();
			}
		});
		
		viewer.addCheckStateListener(new ICheckStateListener()
		{
			public void checkStateChanged(CheckStateChangedEvent event)
			{
				GrepExpression expression = (GrepExpression) (event.getElement());
				expression.setEnabled(event.getChecked());
			}
		});
		
		viewer.addDoubleClickListener(new IDoubleClickListener()
		{
//			@Override
			public void doubleClick(DoubleClickEvent event)
			{
				editExpression();
			}
		});
		
		btnAdd = new Button(composite, SWT.NONE);
		btnAdd.setText("&Add");
		btnAdd.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				addExpression();
			}
		});

		btnAdd.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, false, false));
		
		btnCopy = new Button(composite, SWT.NONE);
		btnCopy.setText("&Copy");
		btnCopy.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				copyExpression();
			}
		});

		btnCopy.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, false, false));
		
		btnEdit = new Button(composite, SWT.NONE);
		btnEdit.setText("&Edit");
		btnEdit.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				editExpression();
			}
		});

		btnEdit.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, false, false));
		
		btnRemove = new Button(composite, SWT.NONE);
		btnRemove.setText("&Remove");
		btnRemove.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				removeExpressions();
			}
		});

		btnRemove.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, false, false));
		
		btnUp = new Button(composite, SWT.NONE);
		btnUp.setText("&Up");
		btnUp.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				moveExpressionsUp();
			}
		});

		btnUp.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, false, false));
		
		btnDown = new Button(composite, SWT.NONE);
		btnDown.setText("&Down");
		btnDown.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				moveExpressionsDown();
			}
		});

		btnDown.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, false, false));
		
		refreshTable();
		return composite;
	}

	/**
	 * Edits and adds a new expression.
	 */
	public void addExpression()
	{
		addExpression(null);
	}
	
	/**
	 * Edits and adds the specified expression.
	 * 
	 * @param grepExpression Expresion.
	 */
	public void addExpression(String grepExpression)
	{
		GrepExpression expression = new GrepExpression();
		GrepStyle style = new GrepStyle(null, RGBUtil.getRandomRgb(), SWT.NONE);
		List<GrepStyle> styles = new ArrayList<GrepStyle>();
		styles.add(style);
		expression.setStyles(styles);
		expression.setEnabled(true);
		expression.setGrepExpression(grepExpression);
		
		EditExpressionDialog dlg = new EditExpressionDialog(getShell(), expression);
		
		if(dlg.open() == Window.OK)
		{
			settings.addExpressionAsFirst(expression);
			refreshTable();
			viewer.setSelection(new StructuredSelection(expression), true);
		}
	}
	
	/**
	 * Copies the currently selected expression and edits it.
	 */
	public void copyExpression()
	{
		GrepExpression srcExpression = getSelectedExpression();
		
		if(srcExpression == null)
		{
			return;
		}
		
		GrepExpression expression = srcExpression.clone();
		
		EditExpressionDialog dlg = new EditExpressionDialog(getShell(), expression);
		
		if(dlg.open() == Window.OK)
		{
			settings.addExpressionAsFirst(expression);
			refreshTable();
			viewer.setSelection(new StructuredSelection(expression), true);
		}
	}
	
	/**
	 * Edits the selected expression.
	 */
	public void editExpression()
	{
		GrepExpression expression = getSelectedExpression();
		
		if(expression == null)
		{
			return;
		}
		
		EditExpressionDialog dlg = new EditExpressionDialog(getShell(), expression);
		
		if(dlg.open() == Window.OK)
		{
			refreshTable();
		}
	}

	/**
	 * Deletes the selected expression(s).
	 */
	public void removeExpressions()
	{
		for(GrepExpression expression: getSelectedExpressions())
		{
			settings.removeExpression(expression);
		}
		
		refreshTable();
	}

	/**
	 * Moves the selected expression(s) upward.
	 */
	public void moveExpressionsUp()
	{
		List<GrepExpression> selection = getSelectedExpressions();
		List<GrepExpression> expressions = settings.getExpressions();
		
		if(expressions.indexOf(selection.get(0)) == 0)
		{
			return;
		}
		
		for(GrepExpression expression: selection)
		{
			int i = expressions.indexOf(expression);
			expressions.remove(expression);
			expressions.add(i - 1, expression);
		}
		
		int[] indices = table.getSelectionIndices();
		
		for(int i = 0; i < indices.length; i++)
		{
			indices[i]--;
		}
		
		settings.setExpressions(expressions);
		refreshTable();
		
		table.setSelection(indices);
		refreshButtons();
	}

	/**
	 * Moves the selected expression(s) downward.
	 */
	public void moveExpressionsDown()
	{
		List<GrepExpression> selection = getSelectedExpressions();
		List<GrepExpression> expressions = settings.getExpressions();
		
		if(expressions.indexOf(selection.get(selection.size() - 1)) == expressions.size() - 1)
		{
			return;
		}
		
		for(GrepExpression expression: selection)
		{
			int i = expressions.indexOf(expression);
			expressions.remove(expression);
			expressions.add(i + 1, expression);
		}
		
		int[] indices = table.getSelectionIndices();
		
		for(int i = 0; i < indices.length; i++)
		{
			indices[i]++;
		}
		
		settings.setExpressions(expressions);
		refreshTable();
		
		table.setSelection(indices);
		refreshButtons();
	}
	
	/**
	 * Returns the selected expression.
	 * 
	 * @return Selected expression, <code>null</code> if no expressions or
	 *     multiple expressions are selected.
	 */
	private GrepExpression getSelectedExpression()
	{
		ISelection selection = viewer.getSelection();
		
		if(!selection.isEmpty() && selection instanceof IStructuredSelection)
		{
			IStructuredSelection ssel = (IStructuredSelection) selection;
			
			if(ssel.toList().size() > 1)
			{
				return null;
			}
			else
			{
				return (GrepExpression) ((IStructuredSelection)selection).getFirstElement();
			}
		}
		else
		{
			return null;
		}
	}

	/**
	 * Returns the selected expressions.
	 * 
	 * @return A list containing the selected expressions.
	 */
	@SuppressWarnings("unchecked")
	private List<GrepExpression> getSelectedExpressions()
	{
		ISelection selection = viewer.getSelection();
		
		if(!selection.isEmpty() && selection instanceof IStructuredSelection)
		{
			IStructuredSelection ssel = (IStructuredSelection) selection;
			List<GrepExpression> expressions = new ArrayList<GrepExpression>();
			
			for(Iterator itr = ssel.iterator(); itr.hasNext();)
			{
				expressions.add((GrepExpression) itr.next());
			}
			
			return expressions;
		}
		else
		{
			return new ArrayList<GrepExpression>();
		}
	}
	
	/**
	 * Refreshes the table.
	 */
	protected void refreshTable()
	{
		viewer.refresh();
	
		for(TableItem item: table.getItems())
		{
			item.setChecked(((GrepExpression)item.getData()).isEnabled());
		}
		
		refreshButtons();
	}
	
	/**
	 * Applies settings.
	 */
	public void apply()
	{
		Activator plugin = Activator.getDefault();
		plugin.setSettings(settings);
	}
	
	/**
	 * Applies settings when the Ok button was clicked.
	 */
	@Override
	public void okPressed()
	{
		apply();
		super.okPressed();
	}

	/**
	 * Applies settings when the Apply button was clicked.
	 * 
	 * @see org.eclipse.jface.dialogs.Dialog#buttonPressed(int)
	 */
	@Override
	protected void buttonPressed(int buttonId)
	{
		if(buttonId == APPLY_ID)
		{
			apply();
		}
		else
		{
			super.buttonPressed(buttonId);
		}
	}

	/**
	 * Enables/disables buttons depending on the current selection.
	 */
	private void refreshButtons()
	{
		int[] selection = table.getSelectionIndices();
		
		btnCopy.setEnabled(selection.length == 1);
		btnEdit.setEnabled(selection.length == 1);
		btnRemove.setEnabled(selection.length > 0);
		btnUp.setEnabled(selection.length > 0 && selection[0] > 0);
		btnDown.setEnabled(selection.length > 0 && selection[selection.length - 1] < table.getItemCount() - 1);
	}
}	
