/*******************************************************************************
 * Copyright (c) 2008 Marian Schedenig
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marian Schedenig - initial API and implementation
 *******************************************************************************/ 

package com.musgit.eclipse.grepconsole;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import org.eclipse.core.runtime.Preferences;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

import com.musgit.eclipse.grepconsole.adapters.GrepExpressionAdapter;
import com.musgit.eclipse.grepconsole.model.GrepConsoleSettings;
import com.musgit.eclipse.grepconsole.model.GrepConsoleSettingsXmlConverter;
import com.musgit.eclipse.grepconsole.model.GrepExpression;
import com.musgit.eclipse.grepconsole.participants.GrepPageParticipant;

/**
 * The activator class for the Grep Console plugin. Controls plugin lifecycle
 * and stores common data shared by the plugin's instances.
 * 
 * @author msched
 */
public class Activator extends AbstractUIPlugin
{
	/** Plugin ID. */
	public static final String PLUGIN_ID = "com.musgit.eclipse.grepconsole";

	/** Key for the settings XML string in the preference storage. */
	private static final String PREFS_KEY = "settings";

	/** Plugin instance. */
	private static Activator plugin;

	/** Grep Console oldSettings. TODO: Store multiple persistent oldSettings. */
	private GrepConsoleSettings settings;

	/** Stores the page participants for the consoles. This allows us to find
	 *  the participant for a specific console later on (e.g. when a refresh
	 *  is required). */
	private Map<StyledText, GrepPageParticipant> viewers = new HashMap<StyledText, GrepPageParticipant>();

	/** Maps RGB values to cached colour instances. */
	private Map<RGB, Color> colors = new HashMap<RGB, Color>();
	
	/** List of expression adapters created from the grep expressions. */
	private List<GrepExpressionAdapter> expressionAdapters;
	
	/**
	 * Creates a new instance.
	 */
	public Activator()
	{
		plugin = this;
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception
	{
		super.start(context);
		
		loadSettings();
		
		if(settings == null)
		{
			settings = new GrepConsoleSettings();
		}
		
		refreshAdapters();
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception
	{
		plugin = null;
		clearColors();
		
		super.stop(context);
	}
	
	/**
	 * Returns the shared instance.
	 * 
	 * @return Shared instance.
	 */
	public static Activator getDefault()
	{
		return plugin;
	}

	/**
	 * Returns the settings instance.
	 * 
	 * @return Settings.
	 */
	public GrepConsoleSettings getSettings()
	{
		return settings;
	}

	/**
	 * Sets the page participant for a specific viewer. This does not actually
	 * register the participant on the viewer, it just records the connection in
	 * the viewer map.
	 * 
	 * @param viewer Viewer.
	 * @param participant Participant.
	 */
	public void setViewer(StyledText viewer, GrepPageParticipant participant)
	{
		viewers.put(viewer, participant);
	}

	/**
	 * Removes the specified page participant from all viewers using it.
	 *  
	 * @param participant This participant will be removed.
	 */
	public void removeViewers(GrepPageParticipant participant)
	{
		Set<StyledText> remove = new HashSet<StyledText>();
		
		for(Entry<StyledText, GrepPageParticipant> entry: viewers.entrySet())
		{
			if(entry.getValue() == participant)
			{
				remove.add(entry.getKey());
			}
		}
		
		for(StyledText viewer: remove)
		{
			viewers.remove(viewer);
		}
	}

	/**
	 * Refreshes all viewers.
	 */
	public void refreshViewers()
	{
		for(StyledText viewer: viewers.keySet())
		{
			// FIXME: force full refresh
			viewer.redraw();
		}
	}

	/**
	 * Stores new Grep Console settings and updates all viewers.
	 * 
	 * @param oldSettings New settings.
	 */
	public void setSettings(GrepConsoleSettings settings)
	{
		this.settings = settings.clone();
		refreshAdapters();
		refreshViewers();
	}
	
	/**
	 * Returns the cached Colour instance for an RGB value. The Colour instance
	 * is created automatically if it did not yet exist.
	 * 
	 * @param rgb RGB value.
	 * 
	 * @return Matching colour instance.
	 */
	public Color getColor(RGB rgb)
	{
		if(rgb == null)
		{
			return null;
		}
		
		Color color = colors.get(rgb);
		
		if(color == null)
		{
			color = new Color(null, rgb);
			colors.put(rgb, color);
		}
		return color;
	}

	/**
	 * Releases all cached colour instances.
	 */
	public void clearColors()
	{
		for(Color color: colors.values())
		{
			color.dispose();
		}
		
		colors.clear();
	}
	
	/**
	 * Rebuilds the list of expression adapters.
	 */
	private void refreshAdapters()
	{
		expressionAdapters = new ArrayList<GrepExpressionAdapter>();
		clearColors();
		
		for(GrepExpression expression: settings.getExpressions())
		{
			expressionAdapters.add(new GrepExpressionAdapter(this, expression));
		}
	}
	
	/**
	 * Returns the list of expression adapters.
	 * 
	 * @return Expression adapters.
	 */
	public ArrayList<GrepExpressionAdapter> getExpressionAdapters()
	{
		return new ArrayList<GrepExpressionAdapter>(expressionAdapters);
	}
	
	/**
	 * Reads the settings from the preference storage.
	 */
	protected void loadSettings()
	{
		Preferences prefs = Activator.getDefault().getPluginPreferences();
		String xml = prefs.getString(PREFS_KEY);
		
		if(xml != null && xml.length() > 0)
		{
			try
			{
				setSettings(GrepConsoleSettingsXmlConverter.xmlStringToSettings(xml));
			}
			catch (Exception ex)
			{
			}
		}
	}
	
	/**
	 * Saves the settings to the preference store.
	 */
	public void saveSettings()
	{
		Preferences prefs = Activator.getDefault().getPluginPreferences();
		
		try
		{
			prefs.setValue(PREFS_KEY, GrepConsoleSettingsXmlConverter.settingsToXmlString(settings));
			savePluginPreferences();
		}
		catch (Exception ex)
		{
		}
	}
}
