/*******************************************************************************
 * Copyright (c) 2008 Marian Schedenig
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marian Schedenig - initial API and implementation
 *******************************************************************************/ 

package com.musgit.eclipse.grepconsole.editexpression;

import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.VerifyEvent;
import org.eclipse.swt.events.VerifyListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.PaletteData;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.ColorDialog;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;

import com.musgit.eclipse.grepconsole.model.GrepExpression;
import com.musgit.eclipse.grepconsole.model.GrepStyle;
import com.musgit.eclipse.grepconsole.util.RGBUtil;
import com.musgit.eclipse.grepconsole.util.StringUtil;

/**
 * A dialog for editing a grep expression.
 * 
 * @author msched
 */
public class EditExpressionDialog extends Dialog
{
	/** Expression being edited. */
	private GrepExpression expression;
	
	/** List of styles for the current expressions. May contain more entries than
	 *  the expression contains groups, since groups being removed from the
	 *  expression are retained during editing. */
	private java.util.List<GrepStyle> groups = new ArrayList<GrepStyle>();

	/** If <code>true</code>, prevents the current style from being updated when
	 *  input is changed. */
	private boolean dontUpdate = false;
	
	/** GUI controls. */
	private Label lblName;
	private Text txtName;
	private Label lblExpression;
	private Text txtExpression;
	private Label lblGroups;
	private List listGroups;
	private Button cbBold;
	private Button cbItalic;
	private Button cbForeground;
	private Text txtForeground;
	private Button btnForeground;
	private Button cbBackground;
	private Text txtBackground;
	private Button btnBackground;
	private Label lblStyle;
	private ListViewer viewerGroups;
	private GrepStyle currentStyle;
	private Text txtUnderline;
	private Button btnUnderline;
	private Button cbBorder;
	private Text txtBorder;
	private Button btnBorder ;
	private Button cbStrikethrough;
	private Text txtStrikethrough;
	private Button btnStrikethrough;
	private Label labelRise;
	private Spinner spinnerRise;
	private Button cbUnderline;
	
	/**
	 * Creates a new dialog.
	 * 
	 * @param parent Parent shell.
	 * @param expression The expression being edited.
	 */
	public EditExpressionDialog(Shell parent, GrepExpression expression)
	{
		super(parent);

		this.expression = expression;
		
		for(GrepStyle style: expression.getStyles())
		{
			groups.add(style.clone());
		}
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.window.Window#getShellStyle()
	 */
	@Override
	protected int getShellStyle()
	{
		return super.getShellStyle() | SWT.RESIZE;
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
	 */
	@Override
	protected void configureShell(Shell newShell)
	{
		super.configureShell(newShell);
		
		newShell.setText("Edit Grep Expression");
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected Control createDialogArea(Composite parent)
	{
		Composite composite = new Composite(parent, SWT.NONE);
		composite.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		GridLayout layout = new GridLayout(2, false);
		composite.setLayout(layout);
		
		Composite panelTop = new Composite(composite, SWT.NONE);
		panelTop.setLayout(new GridLayout(2, false));
		panelTop.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1));
		
		lblName = new Label(panelTop, SWT.NONE);
		lblName.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		lblName.setText("&Name");
		
		txtName = new Text(panelTop, SWT.BORDER);
		txtName.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		
		lblExpression = new Label(panelTop, SWT.NONE);
		lblExpression.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		lblExpression.setText("&Expression");
		
		txtExpression = new Text(panelTop, SWT.BORDER);
		txtExpression.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		txtExpression.addModifyListener(new ModifyListener()
		{
//			@Override
			public void modifyText(ModifyEvent e)
			{
				refreshGroups();
				refreshButtons();
			}
		});
		
		Composite panelGroups = new Composite(composite, SWT.NONE);
		panelGroups.setLayout(new GridLayout(1, false));
		panelGroups.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		lblGroups = new Label(panelGroups, SWT.NONE);
		lblGroups.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		lblGroups.setText("&Groups:");
		
		listGroups = new List(panelGroups, SWT.BORDER);
		listGroups.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		listGroups.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				selectGroup(listGroups.getSelectionIndex());
			}
		});
		
		viewerGroups = new ListViewer(listGroups);
		viewerGroups.setContentProvider(new GroupsContentProvider());
		
		Composite panelStyle = new Composite(composite, SWT.NONE);
		panelStyle.setLayout(new GridLayout(3, false));
		panelStyle.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, true));
		
		lblStyle = new Label(panelStyle, SWT.NONE);
		lblStyle.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 3, 1));
		lblStyle.setText("Style:");
		
		SelectionListener styleUpdateListener = new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				updateStyle();
			}
		};

		VerifyListener colorVerifyListener = new VerifyListener()
		{
//			@Override
			public void verifyText(VerifyEvent e)
			{
				try
				{
					RGBUtil.stringToRgb(e.text);
				}
				catch(NumberFormatException ex)
				{
					e.doit = false;
				}
			}
		};
		
		cbBold = new Button(panelStyle, SWT.CHECK);
		cbBold.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		cbBold.setText("&Bold");
		cbBold.addSelectionListener(styleUpdateListener);
		
		cbItalic = new Button(panelStyle, SWT.CHECK);
		cbItalic.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false, 2, 1));
		cbItalic.setText("&Italic");
		cbItalic.addSelectionListener(styleUpdateListener);
		
		cbForeground = new Button(panelStyle, SWT.CHECK);
		cbForeground.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		cbForeground.setText("&Foreground");
		cbForeground.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if(cbForeground.getSelection())
				{
					if(txtForeground.getText().isEmpty())
					{
						txtForeground.setText(RGBUtil.rgbToString(RGBUtil.getRandomRgb()));
					}
				}
				
				updateStyle();
			}
		});
		
		txtForeground = new Text(panelStyle, SWT.BORDER);
		txtForeground.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		txtForeground.setTextLimit(6);
		txtForeground.addVerifyListener(colorVerifyListener);
		txtForeground.addModifyListener(new ModifyListener()
		{
//			@Override
			public void modifyText(ModifyEvent e)
			{
				RGB rgb = RGBUtil.stringToRgb(txtForeground.getText());
				
				cbForeground.setSelection(rgb != null);
				updateColorButton(btnForeground, rgb);
				updateStyle();
			}
		});
		
		btnForeground = new Button(panelStyle, SWT.PUSH);
		btnForeground.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		btnForeground.setImage(createColorImage(null));
		btnForeground.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				selectForegroundColor();
			}
		});
		
		cbBackground = new Button(panelStyle, SWT.CHECK);
		cbBackground.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		cbBackground.setText("B&ackground");
		cbBackground.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if(cbBackground.getSelection())
				{
					if(txtBackground.getText().isEmpty())
					{
						txtBackground.setText(RGBUtil.rgbToString(RGBUtil.getRandomRgb()));
					}
				}
				
				updateStyle();
			}
		});
		
		txtBackground = new Text(panelStyle, SWT.BORDER);
		txtBackground.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		txtBackground.setTextLimit(6);
		txtBackground.addVerifyListener(colorVerifyListener);
		txtBackground.addModifyListener(new ModifyListener()
		{
//			@Override
			public void modifyText(ModifyEvent e)
			{
				RGB rgb = RGBUtil.stringToRgb(txtBackground.getText());

				cbBackground.setSelection(rgb != null);
				updateColorButton(btnBackground, rgb);
				updateStyle();
			}
		});
		
		btnBackground = new Button(panelStyle, SWT.PUSH);
		btnBackground.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		btnBackground.setImage(createColorImage(null));
		btnBackground.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				selectBackgroundColor();
			}
		});

		cbUnderline = new Button(panelStyle, SWT.CHECK);
		cbUnderline.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		cbUnderline.setText("&Underline");
		cbUnderline.addSelectionListener(styleUpdateListener);
		
		txtUnderline = new Text(panelStyle, SWT.BORDER);
		txtUnderline.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		txtUnderline.setTextLimit(6);
		txtUnderline.addVerifyListener(colorVerifyListener);
		txtUnderline.addModifyListener(new ModifyListener()
		{
//			@Override
			public void modifyText(ModifyEvent e)
			{
				RGB rgb = RGBUtil.stringToRgb(txtUnderline.getText());
				
				if(rgb != null)
				{
					cbUnderline.setSelection(true);
				}
				
				updateColorButton(btnUnderline, rgb);
				updateStyle();
			}
		});
		
		btnUnderline = new Button(panelStyle, SWT.PUSH);
		btnUnderline.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		btnUnderline.setImage(createColorImage(null));
		btnUnderline.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				selectUnderlineColor();
			}
		});

		cbStrikethrough = new Button(panelStyle, SWT.CHECK);
		cbStrikethrough.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		cbStrikethrough.setText("&Strikethrough");
		cbStrikethrough.addSelectionListener(styleUpdateListener);
		
		txtStrikethrough = new Text(panelStyle, SWT.BORDER);
		txtStrikethrough.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		txtStrikethrough.setTextLimit(6);
		txtStrikethrough.addVerifyListener(colorVerifyListener);
		txtStrikethrough.addModifyListener(new ModifyListener()
		{
//			@Override
			public void modifyText(ModifyEvent e)
			{
				RGB rgb = RGBUtil.stringToRgb(txtStrikethrough.getText());
				
				if(rgb != null)
				{
					cbStrikethrough.setSelection(true);
				}
				
				updateColorButton(btnStrikethrough, rgb);
				updateStyle();
			}
		});
		
		btnStrikethrough = new Button(panelStyle, SWT.PUSH);
		btnStrikethrough.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		btnStrikethrough.setImage(createColorImage(null));
		btnStrikethrough.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				selectStrikethroughColor();
			}
		});
		
		cbBorder = new Button(panelStyle, SWT.CHECK);
		cbBorder.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		cbBorder.setText("B&order");
		cbBorder.addSelectionListener(styleUpdateListener);
		
		txtBorder = new Text(panelStyle, SWT.BORDER);
		txtBorder.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		txtBorder.setTextLimit(6);
		txtBorder.addVerifyListener(colorVerifyListener);
		txtBorder.addModifyListener(new ModifyListener()
		{
//			@Override
			public void modifyText(ModifyEvent e)
			{
				RGB rgb = RGBUtil.stringToRgb(txtBorder.getText());
				
				if(rgb != null)
				{
					cbBorder.setSelection(true);
				}
				
				updateColorButton(btnBorder, rgb);
				updateStyle();
			}
		});
		
		btnBorder = new Button(panelStyle, SWT.PUSH);
		btnBorder.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		btnBorder.setImage(createColorImage(null));
		btnBorder.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				selectBorderColor();
			}
		});

		labelRise = new Label(panelStyle, SWT.NONE);
		labelRise.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		labelRise.setText("&Rise");
		
		spinnerRise = new Spinner(panelStyle, SWT.BORDER);
		spinnerRise.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		spinnerRise.setMinimum(-10);
		spinnerRise.setMaximum(10);
		spinnerRise.addModifyListener(new ModifyListener()
		{
//			@Override
			public void modifyText(ModifyEvent e)
			{
				updateStyle();
			}
		});
		
		refreshExpression();
		refreshGroups();
		listGroups.select(0);
		selectGroup(0);
		
		txtExpression.setFocus();
		
		return composite;
	}

	/**
	 * Refreshes the expression name and expression text fields.
	 */
	protected void refreshExpression()
	{
		txtName.setText(expression.getName() == null ? "" : expression.getName());
		txtExpression.setText(expression.getGrepExpression() == null ? "" : expression.getGrepExpression());
	}
	
	/**
	 * Refreshes the list of groups depending on the currently set expression.
	 */
	protected void refreshGroups()
	{
		viewerGroups.setInput(txtExpression.getText());
	}
	
	/**
	 * Updates the image on a colour selection button.
	 * 
	 * @param btn Button.
	 * @param rgb RGB representation of the colour which shall be displayed on
	 *     the button.
	 */
	private void updateColorButton(Button btn, RGB rgb)
	{
		Image oldImage = btn.getImage();
		btn.setImage(createColorImage(rgb));
		
		if(oldImage != null)
		{
			oldImage.dispose();
		}
	}
	
	/**
	 * Opens a colour dialog and updates GUI elements according to the selected
	 * colour.
	 * 
	 * @param txt Text field which will be set to a RGB string representation of
	 *     the selected colour.
	 * @param checkbox Checkbox which will be enabled when a colour is selected.
	 * @param btn Button which will be set to an image of the selected colour.
	 */
	protected void selectColor(Text txt, Button checkbox, Button btn)
	{
		ColorDialog dlg = new ColorDialog(getShell());
		dlg.setRGB(RGBUtil.stringToRgb(txt.getText()));
		
		RGB color = dlg.open();
		
		if(color != null)
		{
			txt.setText(RGBUtil.rgbToString(color));
			
			if(checkbox != null)
			{
				checkbox.setSelection(true);
			}
			
			if(btn != null)
			{
				updateColorButton(btn, color);				
			}
			
			updateStyle();
		}
	}

	/**
	 * Selects the foreground colour.
	 */
	protected void selectForegroundColor()
	{
		selectColor(txtForeground, cbForeground, btnForeground);
	}

	/**
	 * Selects the background colour.
	 */
	protected void selectBackgroundColor()
	{
		selectColor(txtBackground, cbBackground, btnBackground);
	}

	/**
	 * Selects the underline colour.
	 */
	protected void selectUnderlineColor()
	{
		selectColor(txtUnderline, cbUnderline, btnUnderline);
	}

	/**
	 * Selects the strikeout colour.
	 */
	protected void selectStrikethroughColor()
	{
		selectColor(txtStrikethrough, cbStrikethrough, btnStrikethrough);
	}

	/**
	 * Selects the border colour.
	 */
	protected void selectBorderColor()
	{
		selectColor(txtBorder, cbBorder, btnBorder);
	}

	/**
	 * Creates an image representing the specified colour. If <code>null</code>
	 * is specified, the image's colour area will be transparent.
	 * 
	 * @param color Colour.
	 * 
	 * @return Image.
	 */
	private Image createColorImage(RGB color)
	{
		boolean transparent = false;
		
		if(color == null)
		{
			transparent = true;
			color = new RGB(127, 127, 127);
		}
		
    int height = 16;
    int width = (int)(height * 1.5);

    RGB black = new RGB(0, 0, 0);
    PaletteData dataPalette = new PaletteData(new RGB[] { black, color });
    
    ImageData data = new ImageData(width, height, 2, dataPalette);
    data.transparentPixel = (transparent ? 1 : -1);

    for (int y = 0; y < height; y++)
		{
			for (int x = 0; x < width; x++)
			{
				if (x == 0 || y == 0 || x == width-1 || y == height-1)
				{
					data.setPixel(x, y, 0);
				}
				else
				{
					data.setPixel(x, y, 1);
				}
			}
		}

		return new Image(null, data);
	}

	/**
	 * Called when the user selected a group from the list.
	 * @param index
	 */
	private void selectGroup(int index)
	{
		fillGroups(index + 1);
		setCurrentStyle(groups.get(index));
	}

	/**
	 * Fills the list of groups to match the specified count.
	 * 
	 * @param count Number of groups.
	 */
	private void fillGroups(int count)
	{
		while(groups.size() < count)
		{
			groups.add(new GrepStyle(null, null, SWT.BOLD));
		}
	}

	/**
	 * Updates the current style from the current user input.
	 */
	private void updateStyle()
	{
		if(dontUpdate)
		{
			return;
		}
		
		if(cbBold.getSelection())
		{
			currentStyle.setFontStyle(currentStyle.getFontStyle() | SWT.BOLD);
		}
		else
		{
			currentStyle.setFontStyle(currentStyle.getFontStyle() & ~SWT.BOLD);
		}

		if(cbItalic.getSelection())
		{
			currentStyle.setFontStyle(currentStyle.getFontStyle() | SWT.ITALIC);
		}
		else
		{
			currentStyle.setFontStyle(currentStyle.getFontStyle() & ~SWT.ITALIC);
		}
		
		if(cbForeground.getSelection())
		{
			currentStyle.setForeground(RGBUtil.stringToRgb(txtForeground.getText()));
		}
		else
		{
			currentStyle.setForeground(null);
		}
		
		if(cbBackground.getSelection())
		{
			currentStyle.setBackground(RGBUtil.stringToRgb(txtBackground.getText()));
		}
		else
		{
			currentStyle.setBackground(null);
		}
		
		currentStyle.setUnderline(cbUnderline.getSelection());
		
		if(txtUnderline.getText().trim().length() > 0)
		{
			currentStyle.setUnderlineColor(RGBUtil.stringToRgb(txtUnderline.getText()));
		}
		else
		{
			currentStyle.setUnderlineColor(null);
		}
		
		currentStyle.setStrikeout(cbStrikethrough.getSelection());

		if(txtStrikethrough.getText().trim().length() > 0)
		{
			currentStyle.setStrikeoutColor(RGBUtil.stringToRgb(txtStrikethrough.getText()));
		}
		else
		{
			currentStyle.setStrikeoutColor(null);
		}
		
		currentStyle.setBorder(cbBorder.getSelection());
		
		if(txtBorder.getText().trim().length() > 0)
		{
			currentStyle.setBorderColor(RGBUtil.stringToRgb(txtBorder.getText()));
		}
		else
		{
			currentStyle.setBorderColor(null);
		}
		
		currentStyle.setRise(spinnerRise.getSelection());
	}
	
	/**
	 * Updates the GUI to show the specified style.
	 * 
	 * @param style Style.
	 */
	private void setCurrentStyle(GrepStyle style)
	{
		dontUpdate = true;
		
		try
		{
			currentStyle = style;
			
			cbBold.setSelection((currentStyle.getFontStyle() & SWT.BOLD) > 0);
			cbItalic.setSelection((currentStyle.getFontStyle() & SWT.ITALIC) > 0);
			
			if(currentStyle.getForeground() == null)
			{
				cbForeground.setSelection(false);
				txtForeground.setText("");
				updateColorButton(btnForeground, null);
			}
			else
			{
				cbForeground.setSelection(true);
				txtForeground.setText(RGBUtil.rgbToString(currentStyle.getForeground()));
				updateColorButton(btnForeground, currentStyle.getForeground());
			}
			
			if(currentStyle.getBackground() == null)
			{
				cbBackground.setSelection(false);
				txtBackground.setText("");
				updateColorButton(btnBackground, null);
			}
			else
			{
				cbBackground.setSelection(true);
				txtBackground.setText(RGBUtil.rgbToString(currentStyle.getBackground()));
				updateColorButton(btnBackground, currentStyle.getBackground());
			}
			
			cbUnderline.setSelection(style.isUnderline());
	
			if(currentStyle.getUnderlineColor() == null)
			{
				txtUnderline.setText("");
				updateColorButton(btnUnderline, null);
			}
			else
			{
				txtUnderline.setText(RGBUtil.rgbToString(currentStyle.getUnderlineColor()));
				updateColorButton(btnUnderline, currentStyle.getUnderlineColor());
			}
			
			cbStrikethrough.setSelection(style.isStrikeout());
			
			if(currentStyle.getStrikeoutColor() == null)
			{
				txtStrikethrough.setText("");
				updateColorButton(btnStrikethrough, null);
			}
			else
			{
				txtStrikethrough.setText(RGBUtil.rgbToString(currentStyle.getStrikeoutColor()));
				updateColorButton(btnStrikethrough, currentStyle.getStrikeoutColor());
			}
			
			cbBorder.setSelection(style.isBorder());
			
			if(currentStyle.getBorderColor() == null)
			{
				txtBorder.setText("");
				updateColorButton(btnBorder, null);
			}
			else
			{
				txtBorder.setText(RGBUtil.rgbToString(currentStyle.getBorderColor()));
				updateColorButton(btnBorder, currentStyle.getBorderColor());
			}
			
			spinnerRise.setSelection(currentStyle.getRise());
		}
		finally
		{
			dontUpdate = false;
		}
	}
	
	/**
	 * Updates the expression when the Ok button was clicked.
	 */
	@Override
	protected void okPressed()
	{
		expression.setName(StringUtil.stringOrNull(txtName.getText().trim()));
		expression.setGrepExpression(txtExpression.getText());

		Pattern pattern = Pattern.compile(expression.getGrepExpression());
		Matcher matcher = pattern.matcher("");
		int groupCount = matcher.groupCount();
		
		java.util.List<GrepStyle> styles = new ArrayList<GrepStyle>();
		fillGroups(groupCount + 1);
		
		for(int i = 0; i <= groupCount; i++)
		{
			styles.add(groups.get(i));
		}
		
		expression.setStyles(styles);
		
		super.okPressed();
	}
	
	/**
	 * Updates the Ok button (and other GUI elements) depending on the current
	 * input.
	 */
	protected void refreshButtons()
	{
		boolean ok = checkExpression();
		
		Button btnOk = getButton(IDialogConstants.OK_ID);
		
		if(btnOk != null)
		{
			btnOk.setEnabled(ok);
		}
		
		txtExpression.setForeground(getShell().getDisplay().getSystemColor(ok || txtExpression.getText().trim().length() == 0 ? SWT.COLOR_WIDGET_FOREGROUND : SWT.COLOR_RED));
	}
	
	/**
	 * Checks whether the currently entered expression is valid.
	 * 
	 * @return Whether the expression is valid.
	 */
	protected boolean checkExpression()
	{
		return checkExpression(txtExpression.getText());
	}
	
	/**
	 * Checks whether the specified grep expression is valid.
	 * 
	 * @param expression Grep expression.
	 * 
	 * @return Whether the expression is valid.
	 */
	protected boolean checkExpression(String expression)
	{
		if(expression == null)
		{
			return false;
		}

		expression = expression.trim();
		
		if(expression.length() == 0)
		{
			return false;
		}
		
		try
		{
			Pattern.compile(expression);
		}
		catch(PatternSyntaxException ex)
		{
			return false;
		}
		
		return true;
	}

	/**
	 * Refreshes the buttons after they have been created.
	 * 
	 * @see org.eclipse.jface.dialogs.Dialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected Control createContents(Composite parent)
	{
		Control contents = super.createContents(parent);
		refreshButtons();
		return contents;
	}
}
