/*******************************************************************************
 * Copyright (c) 2008 Marian Schedenig
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marian Schedenig - initial API and implementation
 *******************************************************************************/ 

package com.musgit.eclipse.grepconsole.manageexpressions;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.PaletteData;
import org.eclipse.swt.graphics.RGB;

import com.musgit.eclipse.grepconsole.model.GrepExpression;

/**
 * A label provider for grep expressions. Each expression is represented by
 * its name or expressio (if no name is set) and an image showing the foreground
 * and background colours of the primary style.
 * 
 * @author msched
 *
 */
public class ExpressionsLabelProvider implements ILabelProvider
{
	/** Cached colour images. */
	private Map<RGB[], Image> colorImages = new HashMap<RGB[], Image>();
	
	/**
	 * Clears the colour image cache, disposing all contained images.
	 */
	public void clearColorImages()
	{
		for(Image image: colorImages.values())
		{
			image.dispose();
		}
		
		colorImages.clear();
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#addListener(org.eclipse.jface.viewers.ILabelProviderListener)
	 */
//	@Override
	public void addListener(ILabelProviderListener listener)
	{
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#dispose()
	 */
//	@Override
	public void dispose()
	{
		clearColorImages();
	}
	
	/**
	 * Returns an image showing the specified colour. If no matching image is
	 * found in the cache, it is created and cached automatically.
	 * 
	 * @param color RGB colour.
	 * 
	 * @return Colour image.
	 */
	private Image getColorImage(RGB[] color)
	{
		Image image = colorImages.get(color);
		
		if(image == null)
		{
			image = createColorImage(color);
			colorImages.put(color, image);
		}
		
		return image;
	}

	/**
	 * Creates an image showing the specified colour.
	 * 
	 * @param color RGB colour.
	 * 
	 * @return Colour image.
	 */
	private Image createColorImage(RGB[] color)
	{
    int height = 16;
    int width = (int)(height * 1.5);

    RGB black = new RGB(0, 0, 0);
    RGB left = color[0] == null ? new RGB(255, 255, 255) : color[0];
    RGB right = color[1] == null ? new RGB(255, 255, 255) : color[1];
    
    PaletteData dataPalette = new PaletteData(new RGB[] { black, left, right });
    
    ImageData data = new ImageData(width, height, 2, dataPalette);
    data.transparentPixel = -1;

    for (int y = 0; y < height; y++)
		{
			for (int x = 0; x < width; x++)
			{
				if (x == 0 || y == 0 || x == width-1 || y == height-1)
				{
					data.setPixel(x, y, 0);
				}
				else
				{
					data.setPixel(x, y, (x < width / 2) ? 1 : 2);
				}
			}
		}

		return new Image(null, data);
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#isLabelProperty(java.lang.Object, java.lang.String)
	 */
//	@Override
	public boolean isLabelProperty(Object element, String property)
	{
		return false;
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#removeListener(org.eclipse.jface.viewers.ILabelProviderListener)
	 */
//	@Override
	public void removeListener(ILabelProviderListener listener)
	{
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ILabelProvider#getImage(java.lang.Object)
	 */
//	@Override
	public Image getImage(Object element)
	{
		GrepExpression expression = (GrepExpression) element;
		RGB foreground = expression.getStyles().get(0).getForeground();
		RGB background = expression.getStyles().get(0).getBackground();
		
		return getColorImage(new RGB[]{foreground, background});
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ILabelProvider#getText(java.lang.Object)
	 */
//	@Override
	public String getText(Object element)
	{
		GrepExpression expression = (GrepExpression) element;
		
		if(expression.getName() != null)
		{
			return expression.getName();
		}
		else
		{
			return expression.getGrepExpression();
		}
	}
}
