/*******************************************************************************
 * Copyright (c) 2008 Marian Schedenig
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marian Schedenig - initial API and implementation
 *******************************************************************************/ 

package com.musgit.eclipse.grepconsole.model;

import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.swt.SWT;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.musgit.eclipse.grepconsole.util.RGBUtil;

/**
 * Methods for conversions between Grep Console settings and XML strings.
 * 
 * @author msched
 */
public class GrepConsoleSettingsXmlConverter
{
	/** XML constants. */
	private static final String XML_EXPRESSIONS = "expressions";
	private static final String XML_EXPRESSION = "expression";
	private static final String XML_NAME = "name";
	private static final String XML_GREP_EXPRESSION = "grepExpression";
	private static final String XML_ENABLED = "enabled";
	private static final String XML_STYLE = "style";
	private static final String XML_BOLD = "bold";
	private static final String XML_ITALIC = "italic";
	private static final String XML_FOREGROUND = "foreground";
	private static final String XML_BACKGROUND = "background";
	private static final String XML_UNDERLINE = "underline";
	private static final String XML_UNDERLINE_COLOR = "underlineColor";
	private static final String XML_STRIKEOUT = "strikeout";
	private static final String XML_STRIKEOUT_COLOR = "strikeoutColor";
	private static final String XML_BORDER = "border";
	private static final String XML_BORDER_COLOR = "borderColor";
	private static final String XML_RISE = "rise";

	/**
	 * Converts settings to an XML string.
	 * 
	 * @param settings Settings.
	 * 
	 * @return XML string representing the settings.
	 * 
	 * @throws ParserConfigurationException
	 * @throws TransformerFactoryConfigurationError
	 * @throws TransformerException
	 */
	public static String settingsToXmlString(GrepConsoleSettings settings)
			throws ParserConfigurationException,
			TransformerFactoryConfigurationError, TransformerException
	{
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder = factory.newDocumentBuilder();
		Document doc = builder.newDocument();

		fillDocument(doc, settings);

		Transformer transformer = TransformerFactory.newInstance().newTransformer();
		StringWriter stringWriter = new StringWriter();
		transformer.transform(new DOMSource(doc), new StreamResult(stringWriter));
		StringBuffer buffer = stringWriter.getBuffer();
		return buffer.toString();
	}

	/**
	 * Writes settings to a DOM document.
	 * 
	 * @param doc DOM document.
	 * @param settings Settings.
	 */
	private static void fillDocument(Document doc, GrepConsoleSettings settings)
	{
		Element root = doc.createElement(XML_EXPRESSIONS);
		doc.appendChild(root);

		for (GrepExpression expression : settings.getExpressions())
		{
			Element node = doc.createElement(XML_EXPRESSION);
			fillExpressionNode(doc, node, expression);
			root.appendChild(node);
		}
	}

	/**
	 * Writes a GrepExpression instance to a DOM node.
	 * 
	 * @param doc DOM document.
	 * @param node DOM node.
	 * @param expression Grep expression.
	 */
	private static void fillExpressionNode(Document doc, Element node,
			GrepExpression expression)
	{
		if (!expression.isEnabled())
		{
			node.setAttribute(XML_ENABLED, Boolean.toString(expression.isEnabled()));
		}

		if (expression.getName() != null)
		{
			node.setAttribute(XML_NAME, expression.getName());
		}

		node.setAttribute(XML_GREP_EXPRESSION, expression.getGrepExpression());

		for(GrepStyle style : expression.getStyles())
		{
			Element styleNode = doc.createElement(XML_STYLE);

			if((style.getFontStyle() & SWT.BOLD) != 0)
			{
				styleNode.setAttribute(XML_BOLD, Boolean.toString(true));
			}

			if((style.getFontStyle() & SWT.ITALIC) != 0)
			{
				styleNode.setAttribute(XML_ITALIC, Boolean.toString(true));
			}

			if (style.getForeground() != null)
			{
				styleNode.setAttribute(XML_FOREGROUND, RGBUtil.rgbToString(style
						.getForeground()));
			}

			if(style.getBackground() != null)
			{
				styleNode.setAttribute(XML_BACKGROUND, RGBUtil.rgbToString(style
						.getBackground()));
			}

			if(style.isUnderline())
			{
				styleNode.setAttribute(XML_UNDERLINE, String.valueOf(style.isUnderline()));
				
				if(style.getUnderlineColor() != null)
				{
					styleNode.setAttribute(XML_UNDERLINE_COLOR, RGBUtil.rgbToString(style
							.getUnderlineColor()));
				}
			}
			
			if(style.isStrikeout())
			{
				styleNode.setAttribute(XML_STRIKEOUT, String.valueOf(style.isStrikeout()));
				
				if(style.getStrikeoutColor() != null)
				{
					styleNode.setAttribute(XML_STRIKEOUT_COLOR, RGBUtil.rgbToString(style
							.getStrikeoutColor()));
				}
			}
			
			if(style.isBorder())
			{
				styleNode.setAttribute(XML_BORDER, String.valueOf(style.isBorder()));
				
				if(style.getBorderColor() != null)
				{
					styleNode.setAttribute(XML_BORDER_COLOR, RGBUtil.rgbToString(style
							.getBorderColor()));
				}
			}
			
			if(style.getRise() != 0)
			{
				styleNode.setAttribute(XML_RISE, String.valueOf(style.getRise()));
			}

			node.appendChild(styleNode);
		}
	}

	/**
	 * Reads settings from an XML string.
	 * 
	 * @param xml XML string containing Grep Console settings.
	 * 
	 * @return Settings instance read from the string.
	 * 
	 * @throws ParserConfigurationException
	 * @throws SAXException
	 * @throws IOException
	 */
	public static GrepConsoleSettings xmlStringToSettings(String xml) throws ParserConfigurationException, SAXException, IOException
	{
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

		DocumentBuilder builder = factory.newDocumentBuilder();
		Document doc = builder.parse(new InputSource(new StringReader(xml)));

		return documentToSettings(doc);
	}

	/**
	 * Reads settings from a DOM document.
	 * 
	 * @param doc DOM document string containing Grep Console settings.
	 * 
	 * @return Settings instance read from the document.
	 */
	private static GrepConsoleSettings documentToSettings(Document doc)
	{
		GrepConsoleSettings settings = new GrepConsoleSettings();
		
		Node root = doc.getChildNodes().item(0);
		
		if(XML_EXPRESSIONS.equals(root.getNodeName()))
		{
			readExpressions((Element) root, settings);
		}
		
		return settings;
	}

	/**
	 * Reads grep expressions from a DOM element.
	 * 
	 * @param root DOM element.
	 * @param settings Settings to store the read expressions.
	 */
	private static void readExpressions(Element root, GrepConsoleSettings settings)
	{
		NodeList children = root.getChildNodes();
		
		for(int i = 0; i < children.getLength(); i++)
		{
			Node child = children.item(i);
			
			if(child instanceof Element && XML_EXPRESSION.equals(child.getNodeName()))
			{
				settings.addExpression(readExpression((Element) child));
			}
		}
	}

	/**
	 * Reads a single grep expression from a DOM element.
	 * 
	 * @param element DOM element.
	 * 
	 * @return Expression read from the element.
	 */
	private static GrepExpression readExpression(Element element)
	{
		GrepExpression expression = new GrepExpression();
		
		Boolean b = readBooleanAttribute(element, XML_ENABLED);
		
		if(b == null || b == true)
		{
			expression.setEnabled(true);
		}
		
		String s = element.getAttribute(XML_NAME);
		
		if(s != null && s.length() > 0)
		{
			expression.setName(s);
		}
		
		s = element.getAttribute(XML_GREP_EXPRESSION);
		
		if(s != null && s.length() > 0)
		{
			expression.setGrepExpression(s);
		}
		
		List<GrepStyle> styles = new ArrayList<GrepStyle>();
		NodeList children = element.getChildNodes();
		
		for(int i = 0; i < children.getLength(); i++)
		{
			Node child = children.item(i);
			
			if(child instanceof Element && XML_STYLE.equals(child.getNodeName()))
			{
				styles.add(readStyle((Element)child));
			}
		}
		
		expression.setStyles(styles);
		return expression;
	}

	/**
	 * Reads a style from a DOM element.
	 * 
	 * @param element DOM element.
	 * 
	 * @return Style read from the element.
	 */
	private static GrepStyle readStyle(Element element)
	{
		GrepStyle style = new GrepStyle();
		
		Boolean b = readBooleanAttribute(element, XML_BOLD);
		
		if(b != null && b == true)
		{
			style.setFontStyle(style.getFontStyle() | SWT.BOLD);
		}
		
		b = readBooleanAttribute(element, XML_ITALIC);
		
		if(b != null && b == true)
		{
			style.setFontStyle(style.getFontStyle() | SWT.ITALIC);
		}
		
		String s = element.getAttribute(XML_FOREGROUND);
		
		if(s != null && s.length() > 0)
		{
			style.setForeground(RGBUtil.stringToRgb(s));
		}
		
		s = element.getAttribute(XML_BACKGROUND);
		
		if(s != null && s.length() > 0)
		{
			style.setBackground(RGBUtil.stringToRgb(s));
		}
		
		b = readBooleanAttribute(element, XML_UNDERLINE);
		
		if(b != null)
		{
			style.setUnderline(b);
		}
		
		s = element.getAttribute(XML_UNDERLINE_COLOR);
		
		if(s != null && s.length() > 0)
		{
			style.setUnderlineColor(RGBUtil.stringToRgb(s));
		}
		
		b = readBooleanAttribute(element, XML_STRIKEOUT);
		
		if(b != null)
		{
			style.setStrikeout(b);
		}
		
		s = element.getAttribute(XML_STRIKEOUT_COLOR);
		
		if(s != null && s.length() > 0)
		{
			style.setStrikeoutColor(RGBUtil.stringToRgb(s));
		}
		
		b = readBooleanAttribute(element, XML_BORDER);
		
		if(b != null)
		{
			style.setBorder(b);
		}
		
		s = element.getAttribute(XML_BORDER_COLOR);
		
		if(s != null && s.length() > 0)
		{
			style.setBorderColor(RGBUtil.stringToRgb(s));
		}
		
		Integer i = readIntegerAttribute(element, XML_RISE);
		
		if(i != null)
		{
			style.setRise(i);
		}
		
		return style;
	}

	/**
	 * Reads an integer attribute from a DOM element.
	 * 
	 * @param element DOM element.
	 * @param attribute Attribute name.
	 * 
	 * @return Integer value. <code>null</code> if the attribute is empty.
	 */
	private static Integer readIntegerAttribute(Element element, String attribute)
	{
		String s = element.getAttribute(attribute);
		
		if(s == null || s.length() == 0)
		{
			return null;
		}
		
		return Integer.parseInt(s);
	}

	/**
	 * Reads a boolean attribute from a DOM element.
	 * 
	 * @param element DOM element.
	 * @param attribute Attribute name.
	 * 
	 * @return Boolean value. <code>null</code> if the attribute is empty.
	 */
	private static Boolean readBooleanAttribute(Element element, String attribute)
	{
		String s = element.getAttribute(attribute);
		
		if(s == null || s.length() == 0)
		{
			return null;
		}
		
		return Boolean.parseBoolean(s);
	}
}
