/*******************************************************************************
 * Copyright (c) 2008 Marian Schedenig
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marian Schedenig - initial API and implementation
 *******************************************************************************/ 

package com.musgit.eclipse.grepconsole.model;

import java.util.ArrayList;
import java.util.List;

/**
 * Stores a grep expression and its associated styles.
 * 
 * @author msched
 */
public class GrepExpression
{
	/** Whether the expression is enabled (disabled expressions will be
	 *  ignored). */
	private boolean enabled;
	
	/** An (optional) human-readable name for the expression. */
	private String name;
	
	/** Grep expression string. */
	private String grepExpression;
	
	/** Styles. The first style is used for the entire line, additional styles
	 *  are used for the grep pattern groups with the same index. */
	private List<GrepStyle> styles = new ArrayList<GrepStyle>();

	/**
	 * Creates a new expression.
	 */
	public GrepExpression()
	{
	}
	
	/**
	 * Creates a copy of an existing expression.
	 * 
	 * @param src Source expression.
	 */
	private GrepExpression(GrepExpression src)
	{
		enabled = src.enabled;
		name = src.name;
		grepExpression = src.grepExpression;
		
		for(GrepStyle style: src.styles)
		{
			styles.add(style.clone());
		}
	}

	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	@Override
	public GrepExpression clone()
	{
		return new GrepExpression(this);
	}
	
	/**
	 * Determines whether the expression is enabled.
	 * 
	 * @return Whether the expression is enabled.
	 */
	public boolean isEnabled()
	{
		return enabled;
	}

	/**
	 * Enabled/disables the expression.
	 * 
	 * @param enabled <code>true</code> to enable, <code>false</code> to disable
	 *     the expression.
	 */
	public void setEnabled(boolean enabled)
	{
		this.enabled = enabled;
	}

	/**
	 * Returns the (optional) human-readable name for the expression.
	 * 
	 * @return Name (may be <code>null</code>).
	 */
	public String getName()
	{
		return name;
	}

	/**
	 * Sets the (optional) human-readable name for the expression.
	 * 
	 * @param name Name (may be <code>null</code>). 
	 */
	public void setName(String name)
	{
		this.name = name;
	}

	/**
	 * Returns the grep expression string.
	 * 
	 * @return Grep expression string.
	 */
	public String getGrepExpression()
	{
		return grepExpression;
	}

	/**
	 * Sets the grep expression string.
	 * 
	 * @param grepExpression Grep expression string.
	 */
	public void setGrepExpression(String grepExpression)
	{
		this.grepExpression = grepExpression;
	}

	/**
	 * Returns a copy of the list of styles.
	 * 
	 * @return List of styles.
	 */
	public List<GrepStyle> getStyles()
	{
		return new ArrayList<GrepStyle>(styles);
	}

	/**
	 * Sets the list of styles. The original list is copied.
	 * 
	 * @param styles Styles.
	 */
	public void setStyles(List<GrepStyle> styles)
	{
		this.styles = new ArrayList<GrepStyle>(styles);
	}
}
