/*******************************************************************************
 * Copyright (c) 2008 Marian Schedenig
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marian Schedenig - initial API and implementation
 *******************************************************************************/ 

package com.musgit.eclipse.grepconsole.participants;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Matcher;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.LineStyleEvent;
import org.eclipse.swt.custom.LineStyleListener;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.graphics.Color;

import com.musgit.eclipse.grepconsole.Activator;
import com.musgit.eclipse.grepconsole.adapters.GrepExpressionAdapter;

/**
 * Tests console lines against grep expressions and sets their styles
 * accordingly.
 *  
 * @author msched
 */
public class GrepConsoleStyleListener implements LineStyleListener
{
	/**
	 * Tests a single grep expresion against the specified line and calculates
	 * the correct styles.
	 * 
	 * @param text Line of text.
	 * @param lineOffset Line offset.
	 * @param adapter Grep expression adapter.
	 * 
	 * @return Array of styles if the expression matched, <code>null</code>
	 *     otherwise.
	 */
	protected StyleRange[] calculateStyleRange(String text, int lineOffset, GrepExpressionAdapter adapter)
	{
		Matcher matcher = adapter.getPattern().matcher(text);
		
		if(matcher.matches())
		{
			Color foreground = adapter.getForeground(0);
			Color background = adapter.getBackground(0);
			int fontStyle = adapter.getFontStyle(0);
			boolean underline = adapter.getUnderline(0);
			Color underlineColor = adapter.getUnderlineColor(0);
			boolean strikeout = adapter.getStrikeout(0);
			Color strikeoutColor = adapter.getStrikeoutColor(0);
			boolean border = adapter.getBorder(0);
			Color borderColor = adapter.getBorderColor(0);
			int rise = adapter.getRise(0);
			
			int count = matcher.groupCount();

			if(count == 0)
			{
				StyleRange sr = new StyleRange(lineOffset, text.length(), foreground, background, fontStyle);
				
				sr.underline = underline;
				sr.underlineStyle = SWT.UNDERLINE_SINGLE;
				sr.underlineColor = underlineColor;
				
				sr.strikeout = strikeout;
				sr.strikeoutColor = strikeoutColor;
				
				sr.borderStyle = (border ? SWT.BORDER_SOLID : SWT.NONE);
				sr.borderColor = borderColor;
				
				sr.rise = rise;
				
				return new StyleRange[]{sr};
			}
			else
			{
				List<StyleRange> ranges = new ArrayList<StyleRange>();
				int previous = 0;
				
				for(int i = 1; i <= count; i++)
				{
					String group = matcher.group(i);
					
					if(group == null)
					{
						continue;
					}
					
					int start = matcher.start(i);
					int length = group.length();
					
					if(start > previous)
					{
						ranges.add(new StyleRange(lineOffset + previous, start - previous, foreground, background, fontStyle));
					}
				
					Color groupForeground = adapter.getForeground(i);
					Color groupBackground = adapter.getBackground(i);
					int groupFontStyle = adapter.getFontStyle(i) | fontStyle;
					boolean groupUnderline = adapter.getUnderline(i) | underline;
					Color groupUnderlineColor = adapter.getUnderlineColor(i);
					boolean groupStrikeout = adapter.getStrikeout(i) | strikeout;
					Color groupStrikeoutColor = adapter.getStrikeoutColor(i);
					boolean groupBorder = adapter.getBorder(i) | border;
					Color groupBorderColor = adapter.getBorderColor(i);
					int groupRise = adapter.getRise(i);

					if(groupForeground == null)
					{
						groupForeground = foreground;
					}
					
					if(groupBackground == null)
					{
						groupBackground = background;
					}
					
					if(groupUnderlineColor == null)
					{
						groupUnderlineColor = underlineColor;
					}
					
					if(groupStrikeoutColor == null)
					{
						groupStrikeoutColor = strikeoutColor;
					}
					
					if(groupBorderColor == null)
					{
						groupBorderColor = borderColor;
					}
					
					if(groupRise == 0)
					{
						groupRise = rise;
					}
					
					StyleRange sr = new StyleRange(lineOffset + start, length, groupForeground, groupBackground, groupFontStyle);
					
					sr.underline = groupUnderline;
					sr.underlineStyle = SWT.UNDERLINE_SINGLE;
					sr.underlineColor = groupUnderlineColor;
					
					sr.strikeout = groupStrikeout;
					sr.strikeoutColor = groupStrikeoutColor;
					
					sr.borderStyle = (groupBorder ? SWT.BORDER_SOLID : SWT.NONE);
					sr.borderColor = groupBorderColor;
					
					sr.rise = groupRise;
					
					ranges.add(sr);
					previous = start + length;
				}
				
				if(previous < text.length())
				{
					StyleRange sr = new StyleRange(lineOffset + previous, text.length() - previous, foreground, background, fontStyle);
					
					sr.underline = underline;
					sr.underlineStyle = SWT.UNDERLINE_SINGLE;
					sr.underlineColor = underlineColor;
					
					sr.strikeout = strikeout;
					sr.strikeoutColor = strikeoutColor;
					
					sr.borderStyle = (border ? SWT.BORDER_SOLID : SWT.NONE);
					sr.borderColor = borderColor;
					
					sr.rise = rise;
					
					ranges.add(sr);
				}
				
				return ranges.toArray(new StyleRange[0]);
			}
		}
		
		return null;
	}

	/**
	 * Tests all available grep expressions against a line of text and calculates
	 * the line's styles.
	 * 
	 * @see org.eclipse.swt.custom.LineStyleListener#lineGetStyle(org.eclipse.swt.custom.LineStyleEvent)
	 */
//	@Override
	public void lineGetStyle(LineStyleEvent event)
	{
		if(event.lineText.length() == 0)
		{
			return;
		}
		
		ArrayList<GrepExpressionAdapter> adapters = Activator.getDefault().getExpressionAdapters();
		
		List<StyleRange[]> stylesList = new ArrayList<StyleRange[]>();
		
		for(GrepExpressionAdapter adapter: adapters)
		{
			if(adapter.isEnabled())
			{
				StyleRange[] styles = calculateStyleRange(event.lineText, event.lineOffset, adapter);
				
				if(styles != null)
				{
					stylesList.add(styles);
				}
			}
		}

		if(stylesList.size() == 1)
		{
			event.styles = stylesList.iterator().next();
		}
		else if(!stylesList.isEmpty())
		{
			List<StyleRange> styles = new ArrayList<StyleRange>();
			
			int charIndex = 0;
			int[] styleIndex = new int[stylesList.size()];
			int[] lastCharIndex = new int[styleIndex.length];
			
			Arrays.fill(styleIndex, 0);
			
			while(charIndex < event.lineText.length())
			{
				StyleRange currentStyle = new StyleRange();
				currentStyle.start = event.lineOffset + charIndex;
				
				for(int i = 0; i < stylesList.size(); i++)
				{
					StyleRange sr = stylesList.get(i)[styleIndex[i]];
					lastCharIndex[i] = sr.start - event.lineOffset + sr.length - 1;
					
					currentStyle.fontStyle |= sr.fontStyle;
					
					if(currentStyle.foreground == null && sr.foreground != null)
					{
						currentStyle.foreground = sr.foreground;
					}
					
					if(currentStyle.background == null && sr.background != null)
					{
						currentStyle.background = sr.background;
					}

					if(currentStyle.underline == false && sr.underline == true)
					{
						currentStyle.underline = sr.underline;
						currentStyle.underlineColor = sr.underlineColor;
					}
					
					if(currentStyle.strikeout == false && sr.strikeout == true)
					{
						currentStyle.strikeout = sr.strikeout;
						currentStyle.strikeoutColor = sr.strikeoutColor;
					}
					
					if(currentStyle.borderStyle == SWT.NONE && sr.borderStyle != SWT.NONE)
					{
						currentStyle.borderStyle = sr.borderStyle;
						currentStyle.borderColor = sr.borderColor;
					}
					
					if(currentStyle.rise == 0 && sr.rise != 0)
					{
						currentStyle.rise = sr.rise;
					}
					
					int length = lastCharIndex[i] - charIndex + 1;
					
					if(currentStyle.length == 0 || length < currentStyle.length)
					{
						currentStyle.length = length;
					}
				}
				
				styles.add(currentStyle);
				charIndex += currentStyle.length;
				
				for(int i = 0; i < stylesList.size(); i++)
				{
					if(charIndex > lastCharIndex[i])
					{
						styleIndex[i]++;
					}
				}
			}
			
			event.styles = styles.toArray(new StyleRange[0]);
		}
	}
}
